@@ -1,5 +1,6 @@
 package Mojo::RabbitMQ::Client;
 use Mojo::Base 'Mojo::EventEmitter';
+
 use Carp qw(croak confess);
 use Mojo::URL;
 use Mojo::Home;
@@ -8,7 +9,7 @@ use Mojo::Parameters;
 use Mojo::Promise;
 use Mojo::Util qw(url_unescape dumper);
 use List::Util qw(none);
-use Scalar::Util qw(blessed);
+use Scalar::Util qw(blessed weaken);
 use File::Basename 'dirname';
 use File::ShareDir qw(dist_file);
 
@@ -20,7 +21,7 @@ use Mojo::RabbitMQ::Client::LocalQueue;
 require Mojo::RabbitMQ::Client::Consumer;
 require Mojo::RabbitMQ::Client::Publisher;
 
-our $VERSION = "0.2.1";
+our $VERSION = "0.3.1";
 
 use constant DEBUG => $ENV{MOJO_RABBITMQ_DEBUG} // 0;
 
@@ -125,6 +126,7 @@ sub add_channel {
   }
 
   $self->channels->{$id} = $channel->id($id)->client($self);
+  weaken $channel->{client};
 
   return $channel;
 }
@@ -135,9 +137,9 @@ sub acquire_channel_p {
   my $promise = Mojo::Promise->new;
 
   my $channel = Mojo::RabbitMQ::Client::Channel->new();
-  $channel->catch(sub { $promise->reject(@_) });
+  $channel->catch(sub { $promise->reject(@_); undef $promise });
   $channel->on(close => sub { warn "Channel closed" });
-  $channel->on(open => sub { $promise->resolve(@_) });
+  $channel->on(open => sub { $promise->resolve(@_); undef $promise });
 
   $self->open_channel($channel);
 
@@ -349,9 +351,9 @@ sub _connect {
 
       # Connection established
       $stream->on(timeout => sub { $self->_error($id, 'Inactivity timeout') });
-      $stream->on(close => sub { $self->_handle($id, 1) });
+      $stream->on(close => sub { $self && $self->_handle($id, 1) });
       $stream->on(error => sub { $self && $self->_error($id, pop) });
-      $stream->on(read => sub { $self->_read($id, pop) });
+      $stream->on(read => sub { $self && $self->_read($id, pop) });
       $cb->();
     }
   );
@@ -383,9 +385,21 @@ sub _connected {
     'Connection::Start' => sub {
       my $frame = shift;
 
-      my @mechanisms = split /\s/, $frame->method_frame->mechanisms;
-      return $self->emit(error => 'AMQPLAIN is not found in mechanisms')
-        if none { $_ eq 'AMQPLAIN' } @mechanisms;
+      my @server_mechanisms = split /\s/, $frame->method_frame->mechanisms;
+      my $param_mechanism = $self->param('auth_mechanism') // '';
+      my @client_mechanisms = ('AMQPLAIN', 'EXTERNAL');
+      @client_mechanisms = ($param_mechanism) if ($param_mechanism);
+      warn "-- Server mechanisms: @server_mechanisms\n" if DEBUG;
+      warn "-- Client mechanisms: @client_mechanisms\n" if DEBUG;
+      my $mechanism;
+      for my $cand (@client_mechanisms) {
+        if (grep { $_ eq $cand } @server_mechanisms) {
+          $mechanism = $cand;
+          last;
+        }
+      }
+      return $self->emit(error => 'No authentication mechanism could be negotiated')
+        unless $mechanism;
 
       my @locales = split /\s/, $frame->method_frame->locales;
       return $self->emit(error => 'en_US is not found in locales')
@@ -402,7 +416,7 @@ sub _connected {
             information => 'https://github.com/inway/mojo-rabbitmq-client',
             version     => __PACKAGE__->VERSION,
           },
-          mechanism => 'AMQPLAIN',
+          mechanism => $mechanism,
           response  => {LOGIN => $self->user, PASSWORD => $self->pass},
           locale    => 'en_US',
         ),
@@ -751,13 +765,6 @@ Sets username for authorization, by default it's not defined.
 
 Sets user password for authorization, by default it's not defined.
 
-=head2 pass
-
-  my $pass = $client->pass;
-  $client  = $client->pass('secret')
-
-Sets user password for authorization, by default it's not defined.
-
 =head2 host
 
   my $host = $client->host;
@@ -918,7 +925,9 @@ authority file has been provided, or 0x00 otherwise.
 
 =head2 auth_mechanism
 
-Currently only AMQPLAIN is supported, B<so this parameter is ignored>.
+Sets the AMQP authentication mechanism. Defaults to AMQPLAIN. AMQPLAIN and
+EXTERNAL are supported; EXTERNAL will only work if L<Mojo::RabbitMQ::Client> does not need
+to do anything beyond passing along a username and password if specified.
 
 =head2 heartbeat
 
@@ -938,10 +947,12 @@ L<Mojo::RabbitMQ::Client::Channel>, L<Mojo::RabbitMQ::Client::Consumer>, L<Mojo:
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2015-2017, Sebastian Podjasek and others
+Copyright (C) 2015-2019, Sebastian Podjasek and others
 
 Based on L<AnyEvent::RabbitMQ> - Copyright (C) 2010 Masahito Ikuta, maintained by C<< bobtfish@bobtfish.net >>
 
 This program is free software, you can redistribute it and/or modify it under the terms of the Artistic License version 2.0.
 
+Contains AMQP specification (F<shared/amqp0-9-1.stripped.extended.xml>) licensed under BSD-style license.
+
 =cut
